/*
 * Copyright 2014 Higher Frequency Trading
 *
 * http://www.higherfrequencytrading.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package java.lang;

/**
 * A listener for various events in a Thread's life: creation, termination, etc.
 */
public interface ThreadLifecycleListener {

    /**
     * The specified thread is about to be started.
     * @param t the thread which is being started
     */
    void started(Thread t);

    /**
     * The specified thread failed to start.
     * @param t the thread that had a failed start
     */
    void startFailed(Thread t);

    /**
     * The specified thread has been terminated.
     * @param t the thread that has been terminated
     */
    void terminated(Thread t);

}
